import h337 from 'heatmap.js';
import debounce from 'lodash.debounce';

/**
* Detect Element Resize
*
* https://github.com/sdecima/javascript-detect-element-resize
* Sebastian Decima
*
* version: 0.5.3
**/
/*eslint-disable*/
var attachEvent = document.attachEvent,
    stylesCreated = false;

function resetTriggers(element) {
  var triggers = element.__resizeTriggers__,
      expand = triggers.firstElementChild,
      contract = triggers.lastElementChild,
      expandChild = expand.firstElementChild;
  contract.scrollLeft = contract.scrollWidth;
  contract.scrollTop = contract.scrollHeight;
  expandChild.style.width = expand.offsetWidth + 1 + 'px';
  expandChild.style.height = expand.offsetHeight + 1 + 'px';
  expand.scrollLeft = expand.scrollWidth;
  expand.scrollTop = expand.scrollHeight;
}
function checkTriggers(element) {
  return element.offsetWidth != element.__resizeLast__.width || element.offsetHeight != element.__resizeLast__.height;
}

function scrollListener(e) {
  var element = this;
  resetTriggers(this);
  if (this.__resizeRAF__) cancelFrame(this.__resizeRAF__);
  this.__resizeRAF__ = requestFrame(function () {
    if (checkTriggers(element)) {
      element.__resizeLast__.width = element.offsetWidth;
      element.__resizeLast__.height = element.offsetHeight;
      element.__resizeListeners__.forEach(function (fn) {
        fn.call(element, e);
      });
    }
  });
}
if (!attachEvent) {
  var requestFrame = function () {
    var raf = window.requestAnimationFrame || window.mozRequestAnimationFrame || window.webkitRequestAnimationFrame || function (fn) {
      return window.setTimeout(fn, 20);
    };
    return function (fn) {
      return raf(fn);
    };
  }();

  var cancelFrame = function () {
    var cancel = window.cancelAnimationFrame || window.mozCancelAnimationFrame || window.webkitCancelAnimationFrame || window.clearTimeout;
    return function (id) {
      return cancel(id);
    };
  }();

  /* Detect CSS Animations support to detect element display/re-attach */
  var animation = false,
      keyframeprefix = '',
      animationstartevent = 'animationstart',
      domPrefixes = 'Webkit Moz O ms'.split(' '),
      startEvents = 'webkitAnimationStart animationstart oAnimationStart MSAnimationStart'.split(' '),
      pfx = '';
  {
    var elm = document.createElement('fakeelement');
    if (elm.style.animationName !== undefined) {
      animation = true;
    }

    if (animation === false) {
      for (var i = 0; i < domPrefixes.length; i++) {
        if (elm.style[domPrefixes[i] + 'AnimationName'] !== undefined) {
          pfx = domPrefixes[i];
          keyframeprefix = '-' + pfx.toLowerCase() + '-';
          animationstartevent = startEvents[i];
          animation = true;
          break;
        }
      }
    }
  }

  var animationName = 'resizeanim';
  var animationKeyframes = '@' + keyframeprefix + 'keyframes ' + animationName + ' { from { opacity: 0; } to { opacity: 0; } } ';
  var animationStyle = keyframeprefix + 'animation: 1ms ' + animationName + '; ';
}

function createStyles() {
  if (!stylesCreated) {
    //opacity:0 works around a chrome bug https://code.google.com/p/chromium/issues/detail?id=286360
    var css = (animationKeyframes ? animationKeyframes : '') + '.resize-triggers { ' + (animationStyle ? animationStyle : '') + 'visibility: hidden; opacity: 0; } ' + '.resize-triggers, .resize-triggers > div, .contract-trigger:before { content: \" \"; display: block; position: absolute; top: 0; left: 0; height: 100%; width: 100%; overflow: hidden; } .resize-triggers > div { background: #eee; overflow: auto; } .contract-trigger:before { width: 200%; height: 200%; }',
        head = document.head || document.getElementsByTagName('head')[0],
        style = document.createElement('style');

    style.type = 'text/css';
    if (style.styleSheet) {
      style.styleSheet.cssText = css;
    } else {
      style.appendChild(document.createTextNode(css));
    }

    head.appendChild(style);
    stylesCreated = true;
  }
}

function addResizeListener(element, fn) {
  if (attachEvent) element.attachEvent('onresize', fn);else {
    if (!element.__resizeTriggers__) {
      if (getComputedStyle(element).position == 'static') element.style.position = 'relative';
      createStyles();
      element.__resizeLast__ = {};
      element.__resizeListeners__ = [];
      (element.__resizeTriggers__ = document.createElement('div')).className = 'resize-triggers';
      element.__resizeTriggers__.innerHTML = '<div class="expand-trigger"><div></div></div>' + '<div class="contract-trigger"></div>';
      element.appendChild(element.__resizeTriggers__);
      resetTriggers(element);
      element.addEventListener('scroll', scrollListener, true);

      /* Listen for a css animation to detect element display/re-attach */
      animationstartevent && element.__resizeTriggers__.addEventListener(animationstartevent, function (e) {
        if (e.animationName == animationName) resetTriggers(element);
      });
    }
    element.__resizeListeners__.push(fn);
  }
}
function removeResizeListener(element, fn) {
  if (attachEvent) element.detachEvent('onresize', fn);else {
    element.__resizeListeners__.splice(element.__resizeListeners__.indexOf(fn), 1);
    if (!element.__resizeListeners__.length) {
      element.removeEventListener('scroll', scrollListener);
      element.__resizeTriggers__ = !element.removeChild(element.__resizeTriggers__);
    }
  }
}

/*eslint-enable*/

var defineProperty = function (obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
};

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

//

var script = {
  components: {},
  props: {
    options: {
      type: Object,
      default: function _default() {
        return {};
      }
    },
    min: {
      type: Number,
      default: 0
    },
    max: {
      type: Number,
      default: 100
    },
    data: {
      type: Array,
      default: function _default() {
        return [];
      }
    },
    clickDrawable: {
      type: Boolean,
      default: false
    },
    moveDrawable: {
      type: Boolean,
      default: false
    },
    drawValue: {
      type: Number,
      default: 1
    }
  },
  data: function data() {
    return {
      defaultOptions: {}
    };
  },

  computed: {
    mOptions: function mOptions() {
      return _extends({}, this.defaultOptions, this.options);
    }
  },
  mounted: function mounted() {
    var $heatmapContainer = this.$el;
    this.heatmap = h337.create(_extends({
      width: this.$el.clientWidth || 1,
      height: this.$el.clientHeight || 1
    }, this.mOptions, {
      container: $heatmapContainer
    }));
    this.heatmap.setData({
      max: this.max,
      min: this.min,
      data: this.data
    });
    addResizeListener(this.$el, this._handleResize);
  },
  beforeDestroy: function beforeDestroy() {
    removeResizeListener(this.$el, this._handleResize);
  },

  watch: {
    mOptions: function mOptions(val) {
      this.heatmap && this.heatmap.configure(val);
    },
    min: function min(val) {
      this.heatmap && this.heatmap.setDataMin(val);
    },
    max: function max(val) {
      this.heatmap && this.heatmap.setDataMax(val);
    },
    data: function data(val) {
      this.heatmap && this.heatmap.setData({
        max: this.max,
        min: this.min,
        data: val
      });
    }
  },
  methods: {
    _handleResize: function _handleResize() {
      if (!this.$el) return;
      this.heatmap.configure(_extends({
        width: this.$el.clientWidth || 1,
        height: this.$el.clientHeight || 1
      }, this.mOptions));
    },

    _draw: debounce(function (e, type) {
      if (type === 'click' && !this.clickDrawable) return;
      if ((type === 'mousemove' || type === 'touchmove') && !this.moveDrawable) return;
      var x = e.layerX;
      var y = e.layerY;
      if (e.touches) {
        x = e.touches[0].pageX;
        y = e.touches[0].pageY;
      }
      var xField = this.mOptions['xField'] || 'x';
      var yField = this.mOptions['yField'] || 'y';
      var valueField = this.mOptions['valueField'] || 'value';
      if (this.heatmap) {
        var _heatmap$addData;

        this.heatmap.addData((_heatmap$addData = {}, defineProperty(_heatmap$addData, xField, x), defineProperty(_heatmap$addData, yField, y), defineProperty(_heatmap$addData, valueField, this.drawValue), _heatmap$addData));
        this.$emit('change', this.heatmap.getData().data);
      }
    }, 15),
    addData: function addData(val) {
      return this.heatmap && this.heatmap.addData(val);
    },
    getValueAt: function getValueAt(val) {
      return this.heatmap && this.heatmap.getValueAt(val);
    },
    getData: function getData() {
      return this.heatmap && this.heatmap.getData();
    },
    getDataURL: function getDataURL() {
      return this.heatmap && this.heatmap.getDataURL();
    },
    repaint: function repaint() {
      return this.heatmap && this.heatmap.repaint();
    }
  }
};

/* script */
var __vue_script__ = script;

/* template */
var __vue_render__ = function __vue_render__() {
  var _vm = this;
  var _h = _vm.$createElement;
  var _c = _vm._self._c || _h;
  return _c("div", {
    staticClass: "htmp-container",
    on: {
      click: function click($event) {
        _vm._draw($event, "click");
      },
      mousemove: function mousemove($event) {
        _vm._draw($event, "mousemove");
      },
      touchmove: function touchmove($event) {
        _vm._draw($event, "touchmove");
      }
    }
  }, [_vm._t("default")], 2);
};
var __vue_staticRenderFns__ = [];
__vue_render__._withStripped = true;

/* style */
var __vue_inject_styles__ = undefined;
/* scoped */
var __vue_scope_id__ = undefined;
/* module identifier */
var __vue_module_identifier__ = undefined;
/* functional template */
var __vue_is_functional_template__ = false;
/* component normalizer */
function __vue_normalize__(template, style, script$$1, scope, functional, moduleIdentifier, createInjector, createInjectorSSR) {
  var component = (typeof script$$1 === 'function' ? script$$1.options : script$$1) || {};

  // For security concerns, we use only base name in production mode.
  component.__file = "D:\\Workspaces\\heatmapjs-vue\\src\\components\\heatmapjs-vue\\heatmapjs-vue.vue";

  if (!component.render) {
    component.render = template.render;
    component.staticRenderFns = template.staticRenderFns;
    component._compiled = true;

    if (functional) component.functional = true;
  }

  component._scopeId = scope;

  return component;
}
/* style inject */

/* style inject SSR */

var heatmapjsVue = __vue_normalize__({ render: __vue_render__, staticRenderFns: __vue_staticRenderFns__ }, __vue_inject_styles__, __vue_script__, __vue_scope_id__, __vue_is_functional_template__, __vue_module_identifier__, undefined, undefined);

/**
 * Register a component plugin as being loaded. returns true if compoent plugin already registered
 * @param {object} Vue
 * @param {string} Component name
 * @param {object} Component definition
 */
function registerComponent(Vue, name, def) {
  Vue._hm_components_installed = Vue._hm_components_installed || {};
  var loaded = Vue._hm_components_installed[name];
  if (!loaded && def && name) {
    Vue._hm_components_installed[name] = true;
    Vue.component(name, def);
  }
  return loaded;
}

/**
 * Register a group of components as being loaded.
 * @param {object} Vue
 * @param {object} Object of component definitions
 */
function registerComponents(Vue, components) {
  for (var component in components) {
    registerComponent(Vue, component, components[component]);
  }
}

/**
 * Install plugin if window.Vue available
 * @param {object} Plugin definition
 */
function vueUse(VuePlugin) {
  if (typeof window !== 'undefined' && window.Vue) {
    window.Vue.use(VuePlugin);
  }
}

var components = {
  heatmapjsVue: heatmapjsVue
};

var VuePlugin = {
  install: function install(Vue) {
    registerComponents(Vue, components);
  }
};

vueUse(VuePlugin);



var components$1 = /*#__PURE__*/Object.freeze({
  HeatmapjsVue: VuePlugin
});

var VuePlugin$1 = {
  install: function install(Vue) {
    if (Vue._heatmapjs_vue) {
      return;
    }
    Vue._heatmapjs_vue = true;

    // Register component plugins
    for (var plugin in components$1) {
      Vue.use(components$1[plugin]);
    }
  }
};

vueUse(VuePlugin$1);

export default VuePlugin$1;
//# sourceMappingURL=heatmapjs-vue.esm.js.map
